// Copyright (c) 2017, teja. All rights reserved. Use of this source code
// is governed by a BSD-style license that can be found in the LICENSE file.

library mongo_data_store.adapter;

import 'dart:async';
import 'package:jaguar_query/jaguar_query.dart';
import 'package:jaguar_data_store/jaguar_data_store.dart';
import 'package:jaguar_serializer/serializer.dart';

abstract class Idied<IdType> {
  IdType id;
}

class QueryDataStore<IdType, ModelType extends Idied<IdType>, ConnType>
    extends DataStore<ModelType, IdType> {
  final String tableName;

  final MapSerializer<ModelType> serializer;

  final Adapter<ConnType> adapter;

  final String idKey;

  QueryDataStore(this.serializer, this.tableName, this.adapter,
      {this.idKey: "_id"});

  Future<Stream<ModelType>> getAll() async {
    final st = new FindStatement();
    st.from(tableName);
    final Stream<Map> stream = await adapter.find(st);
    return stream.map((Map doc) => serializer.fromMap(doc));
  }

  Future<ModelType> getById(IdType id) async {
    final st = new FindStatement();
    st.from(tableName);
    st.where(eq<IdType>(idKey, id));

    final Map map = await adapter.findOne(st);

    return serializer.fromMap(map);
  }

  Future<IdType> insert(ModelType object) async {
    final Map map = serializer.toMap(object);

    final InsertStatement st = new InsertStatement();
    st.into(tableName);
    map.forEach((String key, dynamic value) {
      st.setValue(key, L.Make(value));
    });

    return await adapter.insert(st);
  }

  Future<Null> updateById(IdType id, ModelType object) async {
    final Map map = serializer.toMap(object);

    final UpdateStatement st = new UpdateStatement();
    st.into(tableName);
    map.forEach((String key, dynamic value) {
      st.setValue(key, L.Make(value));
    });

    st.where(eq<IdType>(idKey, id));

    await adapter.update(st);
  }

  Future<Null> deleteById(IdType id) async {
    final DeleteStatement st = new DeleteStatement();
    st.from(tableName);

    st.where(eq<IdType>(idKey, id));

    await adapter.delete(st);
  }
}
